<?php

namespace Drupal\sitewide_alert\Controller;

use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DrupalDateTime;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Render\Renderer;

/**
 * Class SitewideAlertsController.
 */
class SitewideAlertsController extends ControllerBase {

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * Constructs a new SitewideAlertsController.
   *
   * @param \Drupal\Core\Render\Renderer $renderer
   *   The renderer.
   */
  public function __construct(Renderer $renderer) {
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('renderer')
    );
  }

  /**
   * Load.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Return Hello string.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function load() {
    $response = new CacheableJsonResponse([]);

    $sitewideAlertsJson = ['sitewideAlerts' => []];

    $sitewideAlerts = $this->getSitewideAlerts();

    $viewBuilder = $this->entityTypeManager()->getViewBuilder('sitewide_alert');

    foreach ($sitewideAlerts as $sitewideAlert) {
      $message = $viewBuilder->view($sitewideAlert);

      $sitewideAlertsJson['sitewideAlerts'][] = [
        'uuid' => $sitewideAlert->uuid(),
        'message' => $this->renderer->renderPlain($message),
        'dismissible' => $sitewideAlert->isDismissible(),
        'dismissalIgnoreBefore' => $sitewideAlert->getDismissibleIgnoreBeforeTime(),
        'styleClass' => $sitewideAlert->getStyleClass(),
        'showOnPages' => $sitewideAlert->getPagesToShowOn(),
        'negateShowOnPages' => $sitewideAlert->shouldNegatePagesToShowOn(),
      ];
    }

    // Set the response cache to be dependent on whenever sitewide alerts get updated.
    $cacheableMetadata = (new CacheableMetadata())
      ->setCacheMaxAge(30)
      ->addCacheContexts(['languages'])
      ->setCacheTags(['sitewide_alert_list']);

    $response->addCacheableDependency($cacheableMetadata);
    $response->setData($sitewideAlertsJson);

    // Set the date this response expires so that Drupal's Page Cache will
    // expire this response when the next scheduled alert will be removed.
    // This is needed because Page Cache ignores max age as it does not respect
    // the cache max age. Note that the cache tags will still invalidate this
    // response in the case that new sitewide alerts are added or changed.
    // See Drupal\page_cache\StackMiddleware:storeResponse().
    if ($expireDate = self::soonestExpiringScheduledAlertDateTime($sitewideAlerts)) {
      $response->setExpires($expireDate->getPhpDateTime());
    }

    // Prevent the browser and downstream caches from caching for more than 15 seconds.
    $response->setMaxAge(15);
    $response->setSharedMaxAge(15);

    return $response;
  }

  /**
   * Returns all active sitewide alerts.
   *
   * @return \Drupal\sitewide_alert\Entity\SitewideAlertInterface[]
   *   Array of active sitewide alerts indexed by their ids.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getSitewideAlerts() {
    /** @var \Drupal\sitewide_alert\Entity\SitewideAlertInterface[] $sitewideAlerts */
    $sitewideAlerts = $this->entityTypeManager()->getStorage('sitewide_alert')->loadByProperties(['status' => 1]);

    // Remove any sitewide alerts that are scheduled and it is not time to show them.
    foreach ($sitewideAlerts as $id => $sitewideAlert) {
      if ($sitewideAlert->isScheduled() &&
        !$sitewideAlert->isScheduledToShowAt(new \DateTime('now'))) {
        unset($sitewideAlerts[$id]);
      }
    }

    return $sitewideAlerts;
  }

  /**
   * Determines the datetime of the soonest expiring scheduled alert.
   *
   * @param \Drupal\sitewide_alert\Entity\SitewideAlertInterface[] $sitewideAlerts
   *   Array of sitewide alerts.
   *
   * @return \Drupal\Core\Datetime\DrupalDateTime|null
   *   The datetime of the soonest expiring scheduled alert; null if none of the alerts are scheduled to expire.
   */
  private static function soonestExpiringScheduledAlertDateTime(array $sitewideAlerts) {
    /** @var DrupalDateTime|null $soonestScheduledEndDate */
    $soonestScheduledEndDate = NULL;

    foreach ($sitewideAlerts as $sitewideAlert) {
      if (!$sitewideAlert->isScheduled()) {
        continue;
      }

      if (! $endDateTime = $sitewideAlert->getScheduledEndDateTime()) {
        continue;
      }

      if ($soonestScheduledEndDate === NULL) {
        $soonestScheduledEndDate = $endDateTime;
        continue;
      }

      if ($soonestScheduledEndDate > $endDateTime) {
        $soonestScheduledEndDate = $endDateTime;
      }
    }

    return $soonestScheduledEndDate;
  }

}
